-- Copyright (c) 2011 Cloudera, Inc. All rights reserved.
--
-- Version 12 of the CMF schema. Migration code to be applied on top of the
-- previous version.
--

update SCHEMA_VERSION set VERSION=12;
--
-- Modifications for configuration revisions.
--
-- Each service with configs must be tied to a configuration revision. When the
-- configs are modified, a new revision is created and all non-inherited
-- configs are added to it.
create table CONFIG_REVISIONS (
    REVISION_ID bigint not null,
    SERVICE_ID bigint not null,
    USER_ID bigint,
    CREATED_INSTANT bigint not null,
    MESSAGE longtext not null,
    primary key (REVISION_ID)
) engine=InnoDB;

alter table SERVICES
    add column CONFIG_REVISION_ID bigint;

alter table CONFIGS
    add column REVISION_ID bigint;

--
-- Insert a base revision for each service with existing configs. Automatic
-- incrementing of the revision ID is temporarily enabled to make it easier to
-- create a new revision.
alter table CONFIG_REVISIONS
    modify column REVISION_ID bigint not null auto_increment;
insert into CONFIG_REVISIONS (SERVICE_ID, CREATED_INSTANT, MESSAGE)
    select SERVICE_ID, unix_timestamp(now()) * 1000, 'Base revision'
    from SERVICES s
    where exists
        (select null
         from CONFIGS c
         where c.SERVICE_ID = s.SERVICE_ID);
alter table CONFIG_REVISIONS
    modify column REVISION_ID bigint not null;

--
-- Link each service to its current config revision.
update SERVICES s
    set CONFIG_REVISION_ID=
        (select REVISION_ID
         from CONFIG_REVISIONS r
         where s.SERVICE_ID=r.SERVICE_ID);

--
-- Link each config to the current config revision of its service.
update CONFIGS c
    set REVISION_ID=
        (select REVISION_ID
         from CONFIG_REVISIONS r
         where c.SERVICE_ID=r.SERVICE_ID);

--
-- All data insertion is done. Set up indices and constraints.
--
alter table CONFIGS
    modify column REVISION_ID bigint not null;

alter table SERVICES
    add index FK_SERVICE_CONFIG_REVISION (CONFIG_REVISION_ID),
    add constraint FK_SERVICE_CONFIG_REVISION
    foreign key (CONFIG_REVISION_ID)
    references CONFIG_REVISIONS (REVISION_ID);

alter table CONFIG_REVISIONS
    add index FK_CONFIG_REVISION_SERVICE (SERVICE_ID),
    add constraint FK_CONFIG_REVISION_SERVICE
    foreign key (SERVICE_ID)
    references SERVICES (SERVICE_ID);

alter table CONFIGS
    add index FK_CONFIG_CONFIG_REVISION (REVISION_ID),
    add constraint FK_CONFIG_CONFIG_REVISION
    foreign key (REVISION_ID)
    references CONFIG_REVISIONS (REVISION_ID);
